﻿#region LICENSE
//
// Copyright (c) Microncode.com
// 
// This source is subject to the Microsoft Public License. 
// See https://opensource.org/licenses/ms-pl.html.
// All other rights reserved. 
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
//
#endregion

namespace CSVideoPlayerExampleWinForms
{
    using System;
    using System.Timers;
    using System.Windows.Forms;

    /// <summary>
    /// The main example of using the CSVideoPlayer with Winform. 
    /// </summary>
    public partial class frmMain : Form
    {
        private System.Timers.Timer _SliderTimer;
        private string _mediafile = "";

        int _rotate = 0;
        int _ScaleX = 0;

        /// <summary>
        /// Constructor.
        /// </summary>
        public frmMain()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Open the local media file to play.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdOpen_ClickAsync(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            if (openFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            _mediafile = openFileDialog.FileName;

            videoPlayer.Open(new Uri(_mediafile));
        }

        /// <summary>
        /// Form_Load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmMain_Load(object sender, EventArgs e)
        {
            //Init function, must be call before using any functionality of the library.

            //Set the path of the ffmpeg libraries
            string LibsPath = "";

            //If the LibsPath is empty or "" the library will search for
            //the ffmpeg libraries under the path:
            //[AssemblyDirectory]\CSPlugins\FFmpeg\[x86|x64]\

            videoPlayer.Init("UserName", "RegKey", LibsPath);

            //Get the position with the _SliderTimer
            _SliderTimer = new System.Timers.Timer(200);
            _SliderTimer.Elapsed += OnTimedEvent;
            _SliderTimer.AutoReset = true;
        }

        /// <summary>
        /// Play.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdPlay_Click(object sender, EventArgs e)
        {
            videoPlayer.Play();

            _SliderTimer.Enabled = true;
        }

        /// <summary>
        /// Pause.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdPause_Click(object sender, EventArgs e)
        {
            videoPlayer.Pause();
        }

        /// <summary>
        /// UnPause.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdUnPause_Click(object sender, EventArgs e)
        {
            videoPlayer.Play();
        }

        /// <summary>
        /// Stop.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdStop_Click(object sender, EventArgs e)
        {
            videoPlayer.Stop();
            _SliderTimer.Enabled = false;
            timelineSlider.Value = 0;
        }

        /// <summary>
        /// Rotate +90.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdRotate90_Click(object sender, EventArgs e)
        {
            _rotate += 90;
            videoPlayer.Rotate(_rotate);
        }

        /// <summary>
        /// Rotate -90.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdRotate90Minus_Click(object sender, EventArgs e)
        {
            _rotate -= 90;
            videoPlayer.Rotate(_rotate);
        }

        /// <summary>
        /// Flip.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdFlip_Click(object sender, EventArgs e)
        {
            if (_ScaleX < 0)
                _ScaleX = 1;
            else
                _ScaleX = -1;

            videoPlayer.Flip(_ScaleX);
        }

        /// <summary>
        /// Get the media properties in a new window.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdMediaProperties_Click(object sender, EventArgs e)
        {
            frmMediaProperties frmMediaProperties = new frmMediaProperties();

            frmMediaProperties.GetDetails(_mediafile);

            frmMediaProperties.ShowDialog();
        }

        /// <summary>
        /// Play on full screen.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdFullScreen_Click(object sender, EventArgs e)
        {
            videoPlayer.BringToFront();
            this.FormBorderStyle = FormBorderStyle.None;
            videoPlayer.Dock = DockStyle.Fill;
            this.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Exit the full screen mode on ESC key pressed.
        /// </summary>
        /// <param name="keyData"></param>
        /// <returns></returns>
        protected override bool ProcessDialogKey(Keys keyData)
        {
            if (Form.ModifierKeys == Keys.None && keyData == Keys.Escape)
            {

                this.WindowState = FormWindowState.Normal;
                videoPlayer.Dock = DockStyle.None;
                this.FormBorderStyle = FormBorderStyle.Sizable;

            }
            return base.ProcessDialogKey(keyData);
        }

        /// <summary>
        /// Ratio.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void speedRatioSlider_ValueChanged(object sender, EventArgs e)
        {
            videoPlayer.SpeedRatio(speedRatioSlider.Value);
        }

        /// <summary>
        /// Volume.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void volumeSlider_ValueChanged(object sender, EventArgs e)
        {
            videoPlayer.Volume(volumeSlider.Value);
        }

        #region Events

        /// <summary>
        /// Timer event of the slider.
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        private void OnTimedEvent(Object source, ElapsedEventArgs e)
        {
            this.Invoke(new Action(() =>
            {
                timelineSlider.Value = Convert.ToInt32(videoPlayer.PositionGet().TotalMilliseconds);

            }));
        }

        /// <summary>
        /// Stop move the slider when MouseDown.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timelineSlider_MouseDown(object sender, MouseEventArgs e)
        {
            _SliderTimer.Enabled = false;
        }

        /// <summary>
        /// Seek to the position when MouseUp.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timelineSlider_MouseUp(object sender, MouseEventArgs e)
        {
            int SliderValue = (int)timelineSlider.Value;

            TimeSpan ts = new TimeSpan(0, 0, 0, 0, SliderValue);
            videoPlayer.PositionSet(ts);
            _SliderTimer.Enabled = true;
        }

        /// <summary>
        /// Media opened.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaOpened(object sender, CSVideoPlayer.MediaOpenedEventArgs e)
        {
            Console.WriteLine(videoPlayer.NaturalDuration().TotalMilliseconds);
            timelineSlider.Maximum = Convert.ToInt32(videoPlayer.NaturalDuration().TotalMilliseconds);
        }

        /// <summary>
        /// On MediaChanged
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaChanged(object sender, CSVideoPlayer.MediaOpenedEventArgs e)
        {
            //On MediaChanged
        }

        /// <summary>
        /// On MediaChanging
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaChanging(object sender, CSVideoPlayer.MediaOpeningEventArgs e)
        {
            //On MediaChanging
        }

        /// <summary>
        /// On MediaClosed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaClosed(object sender, EventArgs e)
        {
            //On MediaClosed
        }

        /// <summary>
        /// On MediaEnded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaEnded(object sender, EventArgs e)
        {
            //On MediaEnded
        }

        /// <summary>
        /// On MediaFailed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaFailed(object sender, CSVideoPlayer.MediaFailedEventArgs e)
        {
            //On MediaFailed
        }

        /// <summary>
        /// On MediaInitializing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaInitializing(object sender, CSVideoPlayer.MediaInitializingEventArgs e)
        {
            //On MediaInitializing
        }

        /// <summary>
        /// On MediaOpening
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaOpening(object sender, CSVideoPlayer.MediaOpeningEventArgs e)
        {
            //On MediaOpening
        }

        /// <summary>
        /// On MediaReady
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaReady(object sender, EventArgs e)
        {
            //On MediaReady
        }

        /// <summary>
        /// On MediaStateChanged
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void videoPlayer_MediaStateChanged(object sender, CSVideoPlayer.MediaStateChangedEventArgs e)
        {
            //On MediaStateChanged
        }

        #endregion

        /// <summary>
        /// Order a license.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdOrderLicense_Click(object sender, EventArgs e)
        {
            /// In order to get your own UserName and RegKey and to distribute this library
            /// with your own projects for commercial or any other purpose, please order a license at:
            /// https://www.microncode.com/developers/cs-video-player/?cmd=order

            System.Diagnostics.Process.Start("https://www.microncode.com/developers/cs-video-player/?cmd=order");
        }
    }
}
